#include "gp32.h"
#include "fileio.h"
#include "mp3play.h"

unsigned short *framebuffer;

int py = 10;
void println(char*s)
{
	gp_drawString(20,py,strlen(s),s,0x0000,framebuffer);
	py+=10;
}

#define START_SIZE (128*1024)
#define CHUNK_SIZE (32*1024)

#define SAMPLE_RATE 44100
#define SAMPLE_SIZE ((SAMPLE_RATE/50)*4)
#define SEGMENT_SIZE (SAMPLE_SIZE*2)

#define OUTPUT_BUFSIZE (SAMPLE_SIZE*2)
volatile signed short *out_buffer;
volatile int out_start = 0;
volatile int out_count = 0;

void gp2x_memcpy(void*,void*,unsigned int);

static void MyTIMER4IRQ(void) __attribute__ ((interrupt ("IRQ")));

static void MyTIMER4IRQ(void)
{
	rTCON = BITCLEAR(rTCON,20); // timer4 off
	if (out_count<SAMPLE_SIZE)
	{
		if (out_count>0)
			gp2x_memcpy((void*)out_buffer,(void*)(out_buffer+out_start),out_count*2);
		out_start = 0;
		out_count += MpegAudioDecoder(out_buffer+out_count,SAMPLE_SIZE-out_count,SAMPLE_RATE);
	}
	gp2x_memcpy((void*)gp_getRingSegment(),(void*)(out_buffer+out_start),SAMPLE_SIZE*2);
	out_start += SAMPLE_SIZE;
	out_count -= SAMPLE_SIZE;
	rTCON = BITSET(rTCON,20); // timer4 on
}

static int initsound=1;

void gp_startSound()
{
	long pclk;
	pclk = gp_getPCLK();
	pclk/= 16;
	pclk/= 256;
	pclk/= 50;

	if (initsound)
	{
		gp_initSound(SAMPLE_RATE,16,SEGMENT_SIZE*2); 
		initsound=0; 
	}
	gp_disableIRQ();
	rTCFG0 |= (0xFF<<8);   // Presacler for timer 2,3,4 = 256
	rTCFG1 |= (0x03<<16);  // timer4  1/16
	rTCNTB4 = pclk;
	rTCON  = (0x1<<22) | (0x1<<20); // start timer4, auto reload
	gp_installSWIIRQ(14,MyTIMER4IRQ);
	gp_enableIRQ();
}

//static 
void gp_stopSound()
{
	gp_disableIRQ();
	rTCON = BITCLEAR(rTCON,20); // timer4 off
	gp_removeSWIIRQ(14,MyTIMER4IRQ);
	gp_enableIRQ();
	gp_clearRingbuffer();
}

#define MP3_DIR "dev0:\\MP3\\"

void EnableCache();

void printInfos(char* name)
{
	gp_clearFramebuffer16(framebuffer,0xffff);
	py = 10;
	println(name);
	println("PLAYING");
	py+=10;
	println("A     : NEXT");
	println("B     : PREVIOUS");
	println("L+R   : LOCK/UNLOCK");
	println("START : EXIT");
	py+=10;
}

int main()
{
	DIR dir_list;
	int nbf,cur,lock=0,loading = 0;

	framebuffer = (u16*) FRAMEBUFFER;
	gp_setCpuspeed(100);
	EnableCache();
	//gp_setMMU(framebuffer, ((unsigned char*)framebuffer)+153600-1, 0xFF2);
	gp_initFramebuffer(framebuffer,16,85);

	// geepee problem
	gp_clearFramebuffer16(framebuffer,0xffff);
	py = 10;
	println("PRESS A BUTTON");
	while (gp_getButton());
	gp_clearFramebuffer16(framebuffer,0xffff);
	py = 10;
	println("READING MP3 FOLDER...");

	nbf = smc_dir(MP3_DIR, &dir_list );
	cur = 2;

	out_buffer = (signed short*)malloc(OUTPUT_BUFSIZE+SAMPLE_RATE);
	while (1)
	{
		char buf[64];
		gp_clearFramebuffer16(framebuffer,0xffff);
		py = 10;
		sprintf(buf,"START LOADING %s",dir_list.name[cur]);
		println(buf);
		sprintf(buf,"%s%s",MP3_DIR,dir_list.name[cur]);
		loading = MpegLoadFile(buf,START_SIZE);
		if (loading > 0)
		{
			printInfos(dir_list.name[cur]);
			MpegInitDecoder();
			out_start = 0;
			out_count = MpegAudioDecoder(out_buffer,OUTPUT_BUFSIZE,SAMPLE_RATE);
			gp_startSound();
			if (loading = (loading==START_SIZE))
				println("CONTINUE LOADING...");
			while(1)
			{
				if (loading)
				{
					loading = MpegLoadNextChunk(CHUNK_SIZE);
					if (!loading)
						println("LOADED");
				}
				int key = gp_getButton();
				if ((lock==0) && (key&BUTTON_START))
					gp_Reset();
				else if ((lock==0) && (key&BUTTON_A))
				{
					cur = cur>2?cur-1:nbf-1;
					break;
				}
				else if ((lock==0) && (key&BUTTON_B))
				{
					cur = cur+1<nbf?cur+1:2;
					break;
				}
				else if ((key&(BUTTON_L|BUTTON_R))==(BUTTON_L|BUTTON_R))
				{
					printInfos(dir_list.name[cur]);
					lock ^= 1;
					if (lock)
						println("LOCKED");
					while(gp_getButton())
					{
						if (loading)
						{
							loading = MpegLoadNextChunk(CHUNK_SIZE);
							if (!loading)
								println("LOADED");
						}
					}
				}
			}
			gp_stopSound();
			MpegCloseDecoder();
			while(gp_getButton());
		}
		else
		{
			gp_drawString(20,30,5,"ERROR",0xF800,framebuffer);
			char* err = NULL;
			if (err!=NULL)
				gp_drawString(20,40,strlen(err),err,0xF800,framebuffer);
			while(1)
			{
				int key = gp_getButton();
				if (key&BUTTON_START)
					gp_Reset();
				else if (key&BUTTON_A)
					cur = cur>2?cur-1:nbf-1;
				else if (key&BUTTON_B)
					cur = cur+1<nbf?cur+1:2;
				else
					continue;
				while(gp_getButton());
				break;
			}
		}
	}
}
